@@ -18,7 +18,7 @@ def gather_project_info():
         project = imp.load_source("project", proj_file)
 
     project_dict = dict()
-    for dir in ["data", "analysis", "working"]:
+    for dir in ["data", "analysis", "working", "crash"]:
         path = op.abspath(op.join(lyman_dir, getattr(project, dir + "_dir")))
         project_dict[dir + "_dir"] = path
     project_dict["default_exp"] = project.default_exp
@@ -33,7 +33,7 @@ def gather_project_info():
     return project_dict
 
 
-def gather_experiment_info(exp_name=None, altmodel=None):
+def gather_experiment_info(exp_name=None, altmodel=None, args=None):
     """Import an experiment module and add some formatted information."""
     lyman_dir = os.environ["LYMAN_DIR"]
 
@@ -72,6 +72,7 @@ def gather_experiment_info(exp_name=None, altmodel=None):
     # Save the __doc__ attribute to the dict
     exp_dict["comments"] = "" if exp.__doc__ is None else exp.__doc__
     if altmodel is not None:
+        exp_dict["comments"] += "\n"
         exp_dict["comments"] += "" if alt.__doc__ is None else alt.__doc__
 
     # Check if it looks like this is a partial FOV acquisition
@@ -88,6 +89,10 @@ def gather_experiment_info(exp_name=None, altmodel=None):
     # Build contrasts list if neccesary
     exp_dict["contrast_names"] = [c[0] for c in exp_dict["contrasts"]]
 
+    # Add command line arguments for reproducibility
+    if args is not None:
+        exp_dict["command_line"] = vars(args)
+
     return exp_dict
 
 
@@ -97,10 +102,12 @@ def default_experiment_parameters():
 
         source_template="",
         whole_brain_template="",
+        fieldmap_template="",
         n_runs=0,
 
         TR=2,
         frames_to_toss=0,
+        fieldmap_pe=("y", "y-"),
         temporal_interp=False,
         interleaved=True,
         coreg_init="fsl",
@@ -108,6 +115,7 @@ def default_experiment_parameters():
         intensity_threshold=4.5,
         motion_threshold=1,
         spike_threshold=None,
+        wm_components=6,
         smooth_fwhm=6,
         hpf_cutoff=128,
 
@@ -115,6 +123,8 @@ def default_experiment_parameters():
         condition_names=None,
         regressor_file=None,
         regressor_names=None,
+        confound_sources=["motion"],
+        remove_artifacts=True,
         hrf_model="GammaDifferenceHRF",
         temporal_deriv=False,
         confound_pca=False,
@@ -158,7 +168,8 @@ def determine_subjects(subject_arg=None):
 def determine_engine(args):
     """Read command line args and return Workflow.run() args."""
     plugin_dict = dict(linear="Linear", multiproc="MultiProc",
-                       ipython="IPython", torque="PBS", sge="SGE")
+                       ipython="IPython", torque="PBS", sge="SGE",
+                       slurm="SLURM")
 
     plugin = plugin_dict[args.plugin]
 
