@@ -4,4 +4,4 @@ from registration import create_reg_workflow, spaces
 from fixedfx import create_ffx_workflow
 from mixedfx import create_volume_mixedfx_workflow
 from surfols import create_surface_ols_workflow
-from anatwarp import create_anatwarp_workflow
+from anatwarp import create_fsl_workflow, create_ants_workflow
