@@ -51,6 +51,7 @@ def create_ffx_workflow(name="mni_ffx", space="mni",
     outputnode = Node(IdentityInterface(["flame_results",
                                          "r2_files",
                                          "tsnr_file",
+                                         "mean_file",
                                          "summary_report",
                                          "json_file",
                                          "zstat_report"]),
@@ -82,7 +83,8 @@ def create_ffx_workflow(name="mni_ffx", space="mni",
             [("flame_results", "flame_results")]),
         (ffxsummary, outputnode,
             [("r2_files", "r2_files"),
-             ("tsnr_file", "tsnr_file")]),
+             ("tsnr_file", "tsnr_file"),
+             ("mean_file", "mean_file")]),
         (report, outputnode,
             [("summary_files", "summary_report"),
              ("zstat_files", "zstat_report")]),
@@ -155,6 +157,10 @@ class FFXModel(BaseInterface):
             good_vs = [not np.allclose(d, 0) for d in vs]
             good = np.all([good_cs, good_vs], axis=0)
 
+            # Handle the case where no events occured for this contrast
+            if not good.any():
+                good = np.ones(len(cs), bool)
+
             # Concatenate the cope and varcope data, save only the good frames
             c_data = np.concatenate(cs, axis=-1)[:, :, :, good]
             v_data = np.concatenate(vs, axis=-1)[:, :, :, good]
@@ -194,7 +200,6 @@ class FFXModel(BaseInterface):
                 projcmd = ["mri_vol2surf",
                            "--mov", "%s/zstat1.nii.gz" % contrast,
                            "--reg", self.inputs.reg_file,
-                           "--surf-fwhm", "5",
                            "--hemi", hemi,
                            "--projfrac-avg", "0", "1", ".1",
                            "--o", "%s/%s.zstat1.mgz" % (contrast, hemi)]
@@ -243,6 +248,7 @@ class FFXSummaryOutput(TraitedSpec):
 
     r2_files = OutputMultiPath(File(exists=True))
     tsnr_file = File(exists=True)
+    mean_file = File(exists=True)
 
 
 class FFXSummary(BaseInterface):
@@ -309,6 +315,9 @@ class FFXSummary(BaseInterface):
         tsnr = np.nan_to_num(tsnr)
         self.save_image(tsnr, "tsnr")
 
+        # Save a mean file
+        self.save_image(mean_data, "mean_func")
+
     def save_image(self, data, fname):
         """Save data to the output structure."""
         img = nib.Nifti1Image(data, self.affine, self.header)
@@ -320,6 +329,7 @@ class FFXSummary(BaseInterface):
         outputs["r2_files"] = [op.abspath("r2_full.nii.gz"),
                                op.abspath("r2_main.nii.gz")]
         outputs["tsnr_file"] = op.abspath("tsnr.nii.gz")
+        outputs["mean_file"] = op.abspath("mean_func.nii.gz")
         return outputs
 
 
