@@ -28,7 +28,7 @@ def create_timeseries_model_workflow(name="model", exp_info=None):
         exp_info = lyman.default_experiment_parameters()
 
     # Define constant inputs
-    inputs = ["realign_file", "artifact_file", "timeseries"]
+    inputs = ["realign_file", "nuisance_file", "artifact_file", "timeseries"]
 
     # Possibly add the design and regressor files to the inputs
     if exp_info["design_name"] is not None:
@@ -41,7 +41,8 @@ def create_timeseries_model_workflow(name="model", exp_info=None):
 
     # Set up the experimental design
     modelsetup = MapNode(ModelSetup(exp_info=exp_info),
-                         ["timeseries", "realign_file", "artifact_file"],
+                         ["timeseries", "realign_file",
+                          "nuisance_file", "artifact_file"],
                          "modelsetup")
 
     # For some nodes, make it possible to request extra memory
@@ -51,20 +52,10 @@ def create_timeseries_model_workflow(name="model", exp_info=None):
     modelestimate = MapNode(fsl.FILMGLS(smooth_autocorr=True,
                                         mask_size=5,
                                         threshold=100),
-                            ["design_file", "in_file"],
+                            ["design_file", "in_file", "tcon_file"],
                             "modelestimate")
     modelestimate.plugin_args = mem_request
 
-    # Run the contrast estimation routine
-    contrastestimate = MapNode(fsl.ContrastMgr(),
-                               ["tcon_file",
-                                "dof_file",
-                                "corrections",
-                                "param_estimates",
-                                "sigmasquareds"],
-                               "contrastestimate")
-    contrastestimate.plugin_args = mem_request
-
     # Compute summary statistics about the model fit
     modelsummary = MapNode(ModelSummary(),
                            ["design_matrix_pkl",
@@ -106,6 +97,7 @@ def create_timeseries_model_workflow(name="model", exp_info=None):
     model.connect([
         (inputnode, modelsetup,
             [("realign_file", "realign_file"),
+             ("nuisance_file", "nuisance_file"),
              ("artifact_file", "artifact_file"),
              ("timeseries", "timeseries")]),
         (inputnode, modelestimate,
@@ -113,14 +105,8 @@ def create_timeseries_model_workflow(name="model", exp_info=None):
         (inputnode, saveparams,
             [("timeseries", "in_file")]),
         (modelsetup, modelestimate,
-            [("design_matrix_file", "design_file")]),
-        (modelestimate, contrastestimate,
-            [("dof_file", "dof_file"),
-             ("corrections", "corrections"),
-             ("param_estimates", "param_estimates"),
-             ("sigmasquareds", "sigmasquareds")]),
-        (modelsetup, contrastestimate,
-            [("contrast_file", "tcon_file")]),
+            [("design_matrix_file", "design_file"),
+             ("contrast_file", "tcon_file")]),
         (modelsetup, modelsummary,
             [("design_matrix_pkl", "design_matrix_pkl")]),
         (inputnode, modelsummary,
@@ -142,9 +128,8 @@ def create_timeseries_model_workflow(name="model", exp_info=None):
         (saveparams, outputnode,
             [("json_file", "json_file")]),
         (modelestimate, outputnode,
-            [("results_dir", "results")]),
-        (contrastestimate, outputnode,
-            [("copes", "copes"),
+            [("results_dir", "results"),
+             ("copes", "copes"),
              ("varcopes", "varcopes"),
              ("zstats", "zstats")]),
         (modelsummary, outputnode,
@@ -162,7 +147,7 @@ def create_timeseries_model_workflow(name="model", exp_info=None):
         model.connect(inputnode, "regressor_file",
                       modelsetup, "regressor_file")
     if exp_info["contrasts"]:
-        model.connect(contrastestimate, "zstats",
+        model.connect(modelestimate, "zstats",
                       modelreport, "zstat_files")
         modelreport.iterfield.append("zstat_files")
 
@@ -178,6 +163,7 @@ class ModelSetupInput(BaseInterfaceInputSpec):
     timeseries = File(exists=True)
     design_file = File(exists=True)
     realign_file = File(exists=True)
+    nuisance_file = File(exists=True)
     artifact_file = File(exists=True)
     regressor_file = File(exists=True)
 
@@ -231,12 +217,41 @@ class ModelSetup(BaseInterface):
         else:
             design = None
 
+        # Get confound information to add to the model
+        confounds = []
+        sources = exp_info["confound_sources"]
+        bad_sources = set(sources) - set(["motion", "wm", "brain"])
+        if bad_sources:
+            msg = ("Invalid confound source specification: {}"
+                   .format(list(bad_sources)))
+            raise ValueError(msg)
+
         # Get the motion correction parameters
-        realign = pd.read_csv(self.inputs.realign_file)
-        realign = realign.filter(regex="rot|trans").apply(stats.zscore)
+        if "motion" in sources:
+            realign = pd.read_csv(self.inputs.realign_file)
+            realign = realign.filter(regex="rot|trans").apply(stats.zscore)
+            confounds.append(realign)
+
+        # Get the anatomical nuisance sources
+        nuisance = pd.read_csv(self.inputs.nuisance_file).apply(stats.zscore)
+        if "wm" in sources:
+            wm = nuisance.filter(regex="wm")
+            confounds.append(wm)
+        if "brain" in sources:
+            brain = nuisance["brain"]
+            confounds.append(brain)
+
+        # Combine the different confound sources
+        if confounds:
+            confounds = pd.concat(confounds, axis=1)
+        else:
+            confounds = None
 
         # Get the image artifacts
-        artifacts = pd.read_csv(self.inputs.artifact_file).max(axis=1)
+        if exp_info["remove_artifacts"]:
+            artifacts = pd.read_csv(self.inputs.artifact_file).max(axis=1)
+        else:
+            artifacts = None
 
         # Get the additional model regressors
         if isdefined(self.inputs.regressor_file):
@@ -258,7 +273,7 @@ class ModelSetup(BaseInterface):
                              hrf_model=hrf,
                              ntp=ntp,
                              tr=tr,
-                             confounds=realign,
+                             confounds=confounds,
                              artifacts=artifacts,
                              regressors=regressors,
                              condition_names=exp_info["condition_names"],
@@ -389,17 +404,20 @@ class ModelSummary(BaseInterface):
         ss_full, r2_full = self.compute_r2(yhat_full)
         self.save_image(ss_full, "ssres_full")
         self.save_image(r2_full, "r2_full")
+        del yhat_full, r2_full
 
         # Compute the main model r squared
         yhat_main = self.dot_by_slice(X, pes, "main")
         ss_main, r2_main = self.compute_r2(yhat_main)
         self.save_image(ss_main, "ssres_main")
         self.save_image(r2_main, "r2_main")
+        del yhat_main, r2_main
 
         # Compute the confound model r squared
         yhat_confound = self.dot_by_slice(X, pes, "confound")
         _, r2_confound = self.compute_r2(yhat_confound)
         self.save_image(r2_confound, "r2_confound")
+        del yhat_confound, r2_confound
 
         # Compute and save the residual tSNR
         std = np.sqrt(ss_full / len(y))
@@ -433,8 +451,11 @@ class ModelSummary(BaseInterface):
         return out
 
     def compute_r2(self, yhat):
-
-        ssres = np.sum(np.square(yhat - self.y), axis=-1)
+        """Efficiently compute the coefficient of variation."""
+        ssres = np.zeros_like(self.sstot)
+        n_frames = yhat.shape[-1]
+        for tr in xrange(n_frames):
+            ssres += np.square(yhat[..., tr] - self.y[..., tr])
         r2 = 1 - ssres / self.sstot
         return ssres, r2
 
