@@ -4,7 +4,10 @@ import os.path as op
 import numpy as np
 import pandas as pd
 import nibabel as nib
+import matplotlib as mpl
 import matplotlib.pyplot as plt
+from sklearn import decomposition as decomp
+
 
 from moss.mosaic import Mosaic
 import seaborn as sns
@@ -45,7 +48,10 @@ def create_preprocessing_workflow(name="preproc", exp_info=None):
     in_fields = ["timeseries", "subject_id"]
 
     if exp_info["whole_brain_template"]:
-        in_fields.append("whole_brain_template")
+        in_fields.append("whole_brain")
+
+    if exp_info["fieldmap_template"]:
+        in_fields.append("fieldmap")
 
     inputnode = Node(IdentityInterface(in_fields), "inputs")
 
@@ -53,6 +59,10 @@ def create_preprocessing_workflow(name="preproc", exp_info=None):
     prepare = MapNode(PrepTimeseries(), "in_file", "prep_timeseries")
     prepare.inputs.frames_to_toss = exp_info["frames_to_toss"]
 
+    # Unwarp using fieldmap images
+    if exp_info["fieldmap_template"]:
+        unwarp = create_unwarp_workflow(fieldmap_pe=exp_info["fieldmap_pe"])
+
     # Motion and slice time correct
     realign = create_realignment_workflow(
         temporal_interp=exp_info["temporal_interp"],
@@ -91,6 +101,10 @@ def create_preprocessing_workflow(name="preproc", exp_info=None):
     artifacts.inputs.motion_thresh = exp_info["motion_threshold"]
     artifacts.inputs.spike_thresh = exp_info["spike_threshold"]
 
+    # Extract nuisance variables from anatomical sources
+    confounds = create_confound_extraction_workflow("confounds",
+                                                    exp_info["wm_components"])
+
     # Save the experiment info for this run
     saveparams = MapNode(SaveParameters(exp_info=exp_info),
                          "in_file", "saveparams")
@@ -98,8 +112,6 @@ def create_preprocessing_workflow(name="preproc", exp_info=None):
     preproc.connect([
         (inputnode, prepare,
             [("timeseries", "in_file")]),
-        (prepare, realign,
-            [("out_file", "inputs.timeseries")]),
         (realign, artifacts,
             [("outputs.motion_file", "motion_file")]),
         (realign, coregister,
@@ -127,10 +139,35 @@ def create_preprocessing_workflow(name="preproc", exp_info=None):
             [("outputs.mask_file", "inputs.mask_file")]),
         (filter_rough, artifacts,
             [("outputs.timeseries", "timeseries")]),
+        (filter_rough, confounds,
+            [("outputs.timeseries", "inputs.timeseries")]),
+        (inputnode, confounds,
+            [("subject_id", "inputs.subject_id")]),
+        (skullstrip, confounds,
+            [("outputs.mask_file", "inputs.brain_mask")]),
+        (coregister, confounds,
+            [("outputs.tkreg_mat", "inputs.reg_file")]),
         (inputnode, saveparams,
             [("timeseries", "in_file")]),
         ])
 
+    # Optionally add a connection for unwarping
+    if bool(exp_info["fieldmap_template"]):
+        preproc.connect([
+            (inputnode, unwarp,
+                [("fieldmap", "inputs.fieldmap")]),
+            (prepare, unwarp,
+                [("out_file", "inputs.timeseries")]),
+            (unwarp, realign,
+                [("outputs.timeseries", "inputs.timeseries")])
+        ])
+    else:
+        preproc.connect([
+            (prepare, realign,
+                [("out_file", "inputs.timeseries")]),
+        ])
+
+    # Optionally connect the whole brain template
     if bool(exp_info["whole_brain_template"]):
         preproc.connect([
             (inputnode, coregister,
@@ -146,11 +183,15 @@ def create_preprocessing_workflow(name="preproc", exp_info=None):
                      "realign_report",
                      "mask_report",
                      "artifact_report",
+                     "confound_file",
                      "flirt_affine",
                      "tkreg_affine",
                      "coreg_report",
                      "json_file"]
 
+    if bool(exp_info["fieldmap_template"]):
+        output_fields.append("unwarp_report")
+
     outputnode = Node(IdentityInterface(output_fields), "outputs")
 
     preproc.connect([
@@ -171,16 +212,77 @@ def create_preprocessing_workflow(name="preproc", exp_info=None):
         (filter_rough, outputnode,
             [("outputs.timeseries", "unsmoothed_timeseries"),
              ("outputs.mean_file", "mean_func")]),
+        (confounds, outputnode,
+            [("outputs.confound_file", "confound_file")]),
         (saveparams, outputnode,
             [("json_file", "json_file")]),
         ])
 
+    if bool(exp_info["fieldmap_template"]):
+        preproc.connect([
+            (unwarp, outputnode,
+                [("outputs.report", "unwarp_report")]),
+        ])
+
     return preproc, inputnode, outputnode
 
 
 # =========================================================================== #
 
 
+def create_unwarp_workflow(name="unwarp", fieldmap_pe=("y", "y-")):
+    """Unwarp functional timeseries using reverse phase-blipped images."""
+    inputnode = Node(IdentityInterface(["timeseries", "fieldmap"]), "inputs")
+
+    # Calculate the shift field
+    # Note that setting readout_times to 1 will give a fine
+    # map of the field, but the units will be off
+    # Since we don't write out the map of the field itself, it does
+    # not seem worth it to add another parameter for the readout times.
+    # (It does require that they are the same, but when wouldn't they be?)
+    topup = MapNode(fsl.TOPUP(encoding_direction=fieldmap_pe,
+                              readout_times=[1] * len(fieldmap_pe)),
+                    ["in_file"], "topup")
+
+    # Unwarp the timeseries
+    applytopup = MapNode(fsl.ApplyTOPUP(method="jac", in_index=[1]),
+                         ["in_files",
+                          "in_topup_fieldcoef",
+                          "in_topup_movpar",
+                          "encoding_file"],
+                         "applytopup")
+
+    # Make a figure summarize the unwarping
+    report = MapNode(UnwarpReport(),
+                     ["orig_file", "corrected_file"], "unwarp_report")
+
+    # Define the outputs
+    outputnode = Node(IdentityInterface(["timeseries", "report"]), "outputs")
+
+    # Define and connect the workflow
+    unwarp = Workflow(name)
+    unwarp.connect([
+        (inputnode, topup,
+            [("fieldmap", "in_file")]),
+        (inputnode, applytopup,
+            [("timeseries", "in_files")]),
+        (topup, applytopup,
+            [("out_fieldcoef", "in_topup_fieldcoef"),
+             ("out_movpar", "in_topup_movpar"),
+             ("out_enc_file", "encoding_file")]),
+        (inputnode, report,
+            [("fieldmap", "orig_file")]),
+        (topup, report,
+            [("out_corrected", "corrected_file")]),
+        (applytopup, outputnode,
+            [("out_corrected", "timeseries")]),
+        (report, outputnode,
+            [("out_file", "report")]),
+        ])
+
+    return unwarp
+
+
 def create_realignment_workflow(name="realignment", temporal_interp=True,
                                 TR=2, slice_order="up", interleaved=True):
     """Motion and slice-time correct the timeseries and summarize."""
@@ -478,6 +580,59 @@ def create_filtering_workflow(name="filter",
     return filtering
 
 
+def create_confound_extraction_workflow(name="confounds", wm_components=6):
+    """Extract nuisance variables from anatomical sources."""
+    inputnode = Node(IdentityInterface(["timeseries",
+                                        "brain_mask",
+                                        "reg_file",
+                                        "subject_id"]),
+                     "inputs")
+
+    # Find the subject's Freesurfer segmentation
+    # Grab the Freesurfer aparc+aseg file as an anatomical brain mask
+    getaseg = Node(io.SelectFiles({"aseg": "{subject_id}/mri/aseg.mgz"},
+                                  base_directory=os.environ["SUBJECTS_DIR"]),
+                   "getaseg")
+
+    # Select and erode the white matter to get deep voxels
+    selectwm = Node(fs.Binarize(erode=3, wm=True), "selectwm")
+
+    # Transform the mask into functional space
+    transform = MapNode(fs.ApplyVolTransform(inverse=True,
+                                             interp="nearest"),
+                        ["reg_file", "source_file"],
+                        "transform")
+
+    # Extract eigenvariates of the timeseries from WM and whole brain
+    extract = MapNode(ExtractConfounds(n_components=wm_components),
+                      ["timeseries", "brain_mask", "wm_mask"],
+                      "extract")
+
+    outputnode = Node(IdentityInterface(["confound_file"]), "outputs")
+
+    confounds = Workflow(name)
+    confounds.connect([
+        (inputnode, getaseg,
+            [("subject_id", "subject_id")]),
+        (getaseg, selectwm,
+            [("aseg", "in_file")]),
+        (selectwm, transform,
+            [("binary_file", "target_file")]),
+        (inputnode, transform,
+            [("reg_file", "reg_file"),
+             ("timeseries", "source_file")]),
+        (transform, extract,
+            [("transformed_file", "wm_mask")]),
+        (inputnode, extract,
+            [("timeseries", "timeseries"),
+             ("brain_mask", "brain_mask")]),
+        (extract, outputnode,
+            [("out_file", "confound_file")]),
+        ])
+
+    return confounds
+
+
 # =========================================================================== #
 
 
@@ -517,6 +672,65 @@ class PrepTimeseries(BaseInterface):
     _list_outputs = list_out_file("timeseries.nii.gz")
 
 
+class UnwarpReportInput(BaseInterfaceInputSpec):
+
+    orig_file = File(exists=True)
+    corrected_file = File(exists=True)
+
+
+class UnwarpReport(BaseInterface):
+
+    input_spec = UnwarpReportInput
+    output_spec = SingleOutFile
+
+    def _run_interface(self, runtime):
+
+        # Make a discrete colormap
+        cmap = mpl.colors.ListedColormap(["black", "#d65f5f", "white"])
+
+        # Initialize the figure
+        f, axes = plt.subplots(1, 2, figsize=(9, 2.75),
+                               facecolor="black", edgecolor="black")
+
+        for ax, fname in zip(axes, [self.inputs.orig_file,
+                                    self.inputs.corrected_file]):
+
+            # Combine the frames from this image and plot
+            img = nib.load(fname)
+            ax.imshow(self.combine_frames(img), cmap=cmap, vmin=0, vmax=2)
+            ax.set_axis_off()
+
+        # Save the figure and close
+        f.subplots_adjust(0, 0, 1, 1, 0, 0)
+        f.savefig("unwarping.png", facecolor="black", edgecolor="black")
+        plt.close(f)
+
+        return runtime
+
+    def combine_frames(self, img):
+
+        # Find a value to loosely segment the brain
+        d = img.get_data()
+        counts, bins = np.histogram(d[d > 0], 50)
+        thresh = bins[np.diff(counts) > 0][0]
+
+        # Show the middle slice
+        middle = d.shape[0] // 2
+
+        # Combine a binary mask for each phase direction
+        a = np.rot90(d[middle, ..., 0] > thresh)
+        b = np.rot90(d[middle, ..., 1] > thresh)
+
+        # Make an image showing overlap and divergence
+        c = np.zeros_like(a, int)
+        c[a ^ b] = 1
+        c[a & b] = 2
+
+        return c
+
+    _list_outputs = list_out_file("unwarping.png")
+
+
 class ExtractRealignmentTarget(BaseInterface):
 
     input_spec = SingleInFile
@@ -526,10 +740,9 @@ class ExtractRealignmentTarget(BaseInterface):
 
         # Load the input timeseries
         img = nib.load(self.inputs.in_file)
-        data = img.get_data()
 
         # Extract the target volume
-        targ = self.extract_target(data)
+        targ = self.extract_target(img)
 
         # Save a new 3D image
         targ_img = nib.Nifti1Image(targ,
@@ -539,11 +752,11 @@ class ExtractRealignmentTarget(BaseInterface):
 
         return runtime
 
-    def extract_target(self, data):
+    def extract_target(self, img):
         """Return a 3D array with data from the middle TR."""
-        middle_vol = data.shape[-1] // 2
-        targ = np.empty(data.shape[:-1])
-        targ[:] = data[..., middle_vol]
+        middle_vol = img.shape[-1] // 2
+        targ = np.empty(img.shape[:-1])
+        targ[:] = img.dataobj[..., middle_vol]
 
         return targ
 
@@ -608,22 +821,24 @@ class RealignmentReport(BaseInterface):
             fig, axes = plt.subplots(3, 1, figsize=(9, 5), sharex=True)
 
         # Trim off all but the axis name
-        f = lambda s: s[-1]
+        def axis(s):
+            return s[-1]
 
         # Plot rotations
         pal = sns.color_palette("Reds_d", 3)
-        rot_df = np.rad2deg(df.filter(like="rot")).rename(columns=f)
+        rot_df = np.rad2deg(df.filter(like="rot")).rename(columns=axis)
         rot_df.plot(ax=axes[0], color=pal, lw=1.5)
 
         # Plot translations
         pal = sns.color_palette("Blues_d", 3)
-        trans_df = df.filter(like="trans").rename(columns=f)
+        trans_df = df.filter(like="trans").rename(columns=axis)
         trans_df.plot(ax=axes[1], color=pal, lw=1.5)
 
         # Plot displacement
-        f = lambda s: s[-3:]
+        def ref(s):
+            return s[-3:]
         pal = sns.color_palette("Greens_d", 2)
-        disp_df = df.filter(like="displace").rename(columns=f)
+        disp_df = df.filter(like="displace").rename(columns=ref)
         disp_df.plot(ax=axes[2], color=pal, lw=1.5)
 
         # Label the graphs
@@ -967,3 +1182,48 @@ class ReplaceMean(BaseInterface):
         out_fname = "{}.nii.gz".format(self.inputs.output_name)
         outputs["out_file"] = op.abspath(out_fname)
         return outputs
+
+
+class ExtractConfoundsInput(BaseInterfaceInputSpec):
+
+    timeseries = File(Exists=True)
+    wm_mask = File(Exists=True)
+    brain_mask = File(Exists=True)
+    n_components = traits.Int()
+
+
+class ExtractConfounds(BaseInterface):
+    """Extract nuisance variables from anatomical sources."""
+    input_spec = ExtractConfoundsInput
+    output_spec = SingleOutFile
+
+    def _run_interface(self, runtime):
+
+        # Load the brain images
+        ts_data = nib.load(self.inputs.timeseries).get_data()
+        wm_mask = nib.load(self.inputs.wm_mask).get_data()
+        brain_mask = nib.load(self.inputs.brain_mask).get_data()
+
+        # Set up the output dataframe
+        wm_cols = ["wm_{:d}".format(i)
+                   for i in range(self.inputs.n_components)]
+        cols = wm_cols + ["brain"]
+        index = np.arange(ts_data.shape[-1])
+        out_df = pd.DataFrame(index=index, columns=cols, dtype=np.float)
+
+        # Extract eigenvariates of the white matter timeseries
+        wm_ts = ts_data[wm_mask.astype(bool)].T
+        wm_pca = decomp.PCA(self.inputs.n_components)
+        wm_comp = wm_pca.fit_transform(wm_ts)
+        out_df[wm_cols] = wm_comp
+
+        # Extract the mean whole-brain timeseries
+        brain_ts = ts_data[brain_mask.astype(bool)].mean(axis=0)
+        out_df["brain"] = brain_ts
+
+        # Write out the resulting data to disk
+        out_df.to_csv("nuisance_variables.csv", index=False)
+
+        return runtime
+
+    _list_outputs = list_out_file("nuisance_variables.csv")
