@@ -15,6 +15,7 @@ For normalizations to mni space, two different warps can be used
 """
 import os
 import os.path as op
+import shutil
 import numpy as np
 
 from nipype import Workflow, Node, IdentityInterface
@@ -22,7 +23,8 @@ from nipype.interfaces import fsl
 from nipype.interfaces.base import (BaseInterface,
                                     BaseInterfaceInputSpec,
                                     InputMultiPath, OutputMultiPath,
-                                    TraitedSpec, File, Directory, traits)
+                                    TraitedSpec, File, Directory,
+                                    traits, isdefined)
 
 from lyman.tools import add_suffix, submit_cmdline
 
@@ -30,18 +32,25 @@ spaces = ["epi", "mni"]
 
 
 def create_reg_workflow(name="reg", space="mni",
-                        regtype="model", method="fsl"):
+                        regtype="model", method="fsl",
+                        residual=False, cross_exp=False):
     """Flexibly register files into one of several common spaces."""
 
     # Define the input fields flexibly
     if regtype == "model":
-        fields = ["means", "copes", "varcopes", "sumsquares"]
+        fields = ["copes", "varcopes", "sumsquares"]
     elif regtype == "timeseries":
         fields = ["timeseries"]
-    fields.extend(["masks", "rigids"])
+
+    if cross_exp:
+        fields.extend(["first_rigid"])
+
+    fields.extend(["means", "masks", "rigids"])
 
     if space == "mni":
         fields.extend(["affine", "warpfield"])
+    else:
+        fields.extend(["tkreg_rigid"])
 
     inputnode = Node(IdentityInterface(fields), "inputnode")
 
@@ -51,6 +60,14 @@ def create_reg_workflow(name="reg", space="mni",
     reg_interface = globals()[interface_name]
     transform = Node(reg_interface(method=method), "transform")
 
+    # Sanity check on inputs
+    if regtype == "model" and residual:
+        raise ValueError("residual and regtype=model does not make sense")
+
+    # Set the kind of timeseries
+    if residual:
+        transform.inputs.residual = True
+
     outputnode = Node(IdentityInterface(["out_files"]), "outputnode")
 
     # Define the workflow
@@ -71,6 +88,7 @@ def create_reg_workflow(name="reg", space="mni",
 
 class RegistrationInput(BaseInterfaceInputSpec):
 
+    means = InputMultiPath(File(exists=True))
     masks = InputMultiPath(File(exists=True))
     rigids = InputMultiPath(File(exists=True))
     method = traits.Enum("ants", "fsl")
@@ -78,7 +96,6 @@ class RegistrationInput(BaseInterfaceInputSpec):
 
 class ModelRegInput(BaseInterfaceInputSpec):
 
-    means = InputMultiPath(File(exists=True))
     copes = InputMultiPath(File(exists=True))
     varcopes = InputMultiPath(File(exists=True))
     sumsquares = InputMultiPath(File(exists=True))
@@ -87,6 +104,7 @@ class ModelRegInput(BaseInterfaceInputSpec):
 class TimeseriesRegInput(BaseInterfaceInputSpec):
 
     timeseries = InputMultiPath(File(exists=True))
+    residual = traits.Bool(default=False)
 
 
 class MNIRegInput(BaseInterfaceInputSpec):
@@ -98,6 +116,8 @@ class MNIRegInput(BaseInterfaceInputSpec):
 class EPIRegInput(BaseInterfaceInputSpec):
 
     rigids = InputMultiPath(File(exists=True))
+    first_rigid = File(exists=True)
+    tkreg_rigid = File(exists=True)
 
 
 class MNIModelRegInput(MNIRegInput,
@@ -313,7 +333,10 @@ class EPIModelRegistration(EPIRegistration,
     def _run_interface(self, runtime):
 
         n_runs = len(self.inputs.rigids)
-        first_rigid = self.inputs.rigids[0]
+        if isdefined(self.inputs.first_rigid):
+            first_rigid = self.inputs.first_rigid
+        else:
+            first_rigid = self.inputs.rigids[0]
 
         # Unpack the long file lists to be ordered by run
         copes = self.unpack_files("copes", n_runs)
@@ -351,6 +374,12 @@ class EPIModelRegistration(EPIRegistration,
                 runtime = self.apply_fsl_rigid(runtime, in_file,
                                                out_file, full_rigid)
 
+            # Copy the matrix to go from this space to the anatomy
+            if not i:
+                tkreg_fname = op.basename(self.inputs.tkreg_rigid)
+                out_tkreg = op.join(out_dir, tkreg_fname)
+                shutil.copyfile(self.inputs.tkreg_rigid, out_tkreg)
+
         self.out_files = out_files
         return runtime
 
@@ -378,17 +407,24 @@ class MNITimeseriesRegistration(MNIRegistration,
 
             # Warp the timeseries files
             run_timeseries = self.inputs.timeseries[i]
-            out_timeseries = op.join(out_dir, "timeseries_xfm.nii.gz")
+            name = "res4d" if self.inputs.residual else "timeseries"
+            out_timeseries = op.join(out_dir, name + "_warp.nii.gz")
             run_rigid = self.inputs.rigids[i]
             runtime = warp_func(runtime, run_timeseries,
                                 out_timeseries, run_rigid)
 
             # Warp the mask file
             run_mask = self.inputs.masks[i]
-            out_mask_fname = op.basename(add_suffix(run_mask, "xfm"))
+            out_mask_fname = op.basename(add_suffix(run_mask, "warp"))
             out_mask = op.join(out_dir, out_mask_fname)
             runtime = warp_func(runtime, run_mask, out_mask, run_rigid)
 
+            # Warp the mean file
+            run_mean = self.inputs.means[i]
+            out_mean_fname = op.basename(add_suffix(run_mean, "warp"))
+            out_mean = op.join(out_dir, out_mean_fname)
+            runtime = warp_func(runtime, run_mean, out_mean, run_rigid)
+
         self.out_files = out_files
         return runtime
 
@@ -407,7 +443,10 @@ class EPITimeseriesRegistration(EPIRegistration,
     def _run_interface(self, runtime):
 
         n_runs = len(self.inputs.rigids)
-        first_rigid = self.inputs.rigids[0]
+        if isdefined(self.inputs.first_rigid):
+            first_rigid = self.inputs.first_rigid
+        else:
+            first_rigid = self.inputs.rigids[0]
 
         out_files = []
         for i in range(n_runs):
@@ -425,7 +464,8 @@ class EPITimeseriesRegistration(EPIRegistration,
 
             # Warp the timeseries files
             run_timeseries = self.inputs.timeseries[i]
-            out_timeseries = op.join(out_dir, "timeseries_xfm.nii.gz")
+            name = "res4d" if self.inputs.residual else "timeseries"
+            out_timeseries = op.join(out_dir, name + "_xfm.nii.gz")
             runtime = self.apply_fsl_rigid(runtime, run_timeseries,
                                            out_timeseries, full_rigid)
 
@@ -436,5 +476,18 @@ class EPITimeseriesRegistration(EPIRegistration,
             runtime = self.apply_fsl_rigid(runtime, run_mask,
                                            out_mask, full_rigid)
 
+            # Warp the mean file
+            run_mean = self.inputs.means[i]
+            out_mean_fname = op.basename(add_suffix(run_mean, "xfm"))
+            out_mean = op.join(out_dir, out_mean_fname)
+            runtime = self.apply_fsl_rigid(runtime, run_mean,
+                                           out_mean, full_rigid)
+
+            # Copy the matrix to go from this space to the anatomy
+            if not i:
+                tkreg_fname = op.basename(self.inputs.tkreg_rigid)
+                out_tkreg = op.join(out_dir, tkreg_fname)
+                shutil.copyfile(self.inputs.tkreg_rigid, out_tkreg)
+
         self.out_files = out_files
         return runtime
