@@ -4,8 +4,12 @@ Main execution script for fMRI analysis in the Lyman ecosystem.
 
 """
 import os
+
+# Needed currently to avoid crash in model code
+# Also nipype parallelism doesn't play well with this
+os.environ["MKL_NUM_THREADS"] = "1"
+
 import sys
-import time
 import shutil
 import os.path as op
 from textwrap import dedent
@@ -28,7 +32,7 @@ def main(arglist):
 
     # Get and process specific information
     project = lyman.gather_project_info()
-    exp = lyman.gather_experiment_info(args.experiment, args.altmodel)
+    exp = lyman.gather_experiment_info(args.experiment, args.altmodel, args)
 
     # Set up the SUBJECTS_DIR for Freesurfer
     os.environ["SUBJECTS_DIR"] = project["data_dir"]
@@ -51,11 +55,7 @@ def main(arglist):
     data_dir = project["data_dir"]
     analysis_dir = op.join(project["analysis_dir"], exp_name)
     working_dir = op.join(project["working_dir"], exp_name)
-
-    # Just stick crashdumps in a unique /tmp directory
-    nipype.config.set("execution", "crashdump_dir",
-                      "/tmp/%s-nipype_crashes-%d" % (os.getlogin(),
-                                                     time.time()))
+    nipype.config.set("execution", "crashdump_dir", project["crash_dir"])
 
     # Create symlinks to the preproc directory for altmodels
     if not op.exists(analysis_dir):
@@ -87,7 +87,9 @@ def main(arglist):
     # Collect raw nifti data
     preproc_templates = dict(timeseries=exp["source_template"])
     if exp["partial_brain"]:
-        preproc_templates["whole_brain_template"] = exp["whole_brain_template"]
+        preproc_templates["whole_brain"] = exp["whole_brain_template"]
+    if exp["fieldmap_template"]:
+        preproc_templates["fieldmap"] = exp["fieldmap_template"]
 
     preproc_source = Node(SelectFiles(preproc_templates,
                                       base_directory=project["data_dir"]),
@@ -128,6 +130,7 @@ def main(arglist):
     model_templates = dict(
         timeseries=op.join(model_base, smoothing + "_timeseries.nii.gz"),
         realign_file=op.join(model_base, "realignment_params.csv"),
+        nuisance_file=op.join(model_base, "nuisance_variables.csv"),
         artifact_file=op.join(model_base, "artifacts.csv"),
         )
 
@@ -166,7 +169,10 @@ def main(arglist):
     # ----------------------------------------------------------------------- #
 
     # Is this a model or timeseries registration?
-    regtype = "timeseries" if args.timeseries else "model"
+    regtype = "timeseries" if (args.timeseries or args.residual) else "model"
+
+    # Are we registering across experiments?
+    cross_exp = args.regexp is not None
 
     # Retrieve the right workflow function for registration
     # Get the workflow function dynamically based on the space
@@ -175,7 +181,9 @@ def main(arglist):
     reg, reg_input, reg_output = wf.create_reg_workflow(flow_name,
                                                         space,
                                                         regtype,
-                                                        warp_method)
+                                                        warp_method,
+                                                        args.residual,
+                                                        cross_exp)
 
     # Define a smoothing info node here. Use an iterable so that running
     # with/without smoothing doesn't clobber working directory files
@@ -187,43 +195,65 @@ def main(arglist):
     # Set up the registration inputs and templates
     reg_templates = dict(
         masks="{subject_id}/preproc/run_*/functional_mask.nii.gz",
+        means="{subject_id}/preproc/run_*/mean_func.nii.gz",
                          )
 
     if regtype == "model":
+        # First-level model summary statistic images
         reg_base = "{subject_id}/model/{smoothing}/run_*/"
         reg_templates.update(dict(
-            means="{subject_id}/preproc/run_*/mean_func.nii.gz",
             copes=op.join(reg_base, "cope*.nii.gz"),
             varcopes=op.join(reg_base, "varcope*.nii.gz"),
             sumsquares=op.join(reg_base, "ss*.nii.gz"),
                                   ))
     else:
-        reg_templates.update(dict(
-            timeseries=op.join("{subject_id}/preproc/run_*/",
-                               "{smoothing}_timeseries.nii.gz"),
-                                  ))
+        # Timeseries images
+        if args.residual:
+            ts_file = op.join("{subject_id}/model/{smoothing}/run_*/",
+                              "results/res4d.nii.gz")
+        else:
+            ts_file = op.join("{subject_id}/preproc/run_*/",
+                              "{smoothing}_timeseries.nii.gz")
+        reg_templates.update(dict(timeseries=ts_file))
     reg_lists = reg_templates.keys()
 
+    # Native anatomy to group anatomy affine matrix and warpfield
     if space == "mni":
         aff_ext = "mat" if warp_method == "fsl" else "txt"
         reg_templates["warpfield"] = op.join(data_dir, "{subject_id}",
                                              "normalization/warpfield.nii.gz")
         reg_templates["affine"] = op.join(data_dir, "{subject_id}",
                                           "normalization/affine." + aff_ext)
+    else:
+        if args.regexp is None:
+            tkreg_base = analysis_dir
+        else:
+            tkreg_base = op.join(project["analysis_dir"], args.regexp)
+        reg_templates["tkreg_rigid"] = op.join(tkreg_base,
+                                               "{subject_id}", "preproc",
+                                               "run_1", "func2anat_tkreg.dat")
 
+    # Rigid (6dof) functional-to-anatomical matrices
     rigid_stem = "{subject_id}/preproc/run_*/func2anat_"
     if warp_method == "ants" and space == "mni":
         reg_templates["rigids"] = rigid_stem + "tkreg.dat"
     else:
         reg_templates["rigids"] = rigid_stem + "flirt.mat"
 
+    # Rigid matrix from anatomy to target experiment space
+    if args.regexp is not None:
+        targ_analysis_dir = op.join(project["analysis_dir"], args.regexp)
+        reg_templates["first_rigid"] = op.join(targ_analysis_dir,
+                                               "{subject_id}", "preproc",
+                                               "run_1", "func2anat_flirt.mat")
+
     # Define the registration data source node
     reg_source = Node(SelectFiles(reg_templates,
                                   force_lists=reg_lists,
                                   base_directory=analysis_dir),
                       "reg_source")
 
-    # Registration inutnode
+    # Registration inputnode
     reg_inwrap = tools.InputWrapper(reg, subj_source,
                                     reg_source, reg_input)
     reg_inwrap.connect_inputs()
@@ -241,7 +271,8 @@ def main(arglist):
 
     # Reg has some additional substitutions to strip out iterables
     # and rename the timeseries file
-    reg_outwrap.add_regexp_substitutions([("_smoothing_", "")])
+    reg_subs = [("_smoothing_", "")]
+    reg_outwrap.add_regexp_substitutions(reg_subs)
 
     # Add dummy substitutions for the contasts to make sure the DataSink
     # reruns when the deisgn has changed. This accounts for the problem where
@@ -263,7 +294,8 @@ def main(arglist):
     wf_name = space + "_ffx"
     ffx, ffx_input, ffx_output = wf.create_ffx_workflow(wf_name,
                                                         space,
-                                                        exp["contrast_names"])
+                                                        exp["contrast_names"],
+                                                        exp_info=exp)
 
     ext = "_warp.nii.gz" if space == "mni" else "_xfm.nii.gz"
     ffx_base = op.join("{subject_id}/reg", space, "{smoothing}/run_*")
@@ -284,8 +316,9 @@ def main(arglist):
         reg = op.join(os.environ["FREESURFER_HOME"],
                       "average/mni152.register.dat")
     else:
-        bg = "{subject_id}/preproc/run_1/mean_func.nii.gz"
-        reg = "{subject_id}/preproc/run_1/func2anat_tkreg.dat"
+        reg_dir = "{subject_id}/reg/epi/{smoothing}/run_1"
+        bg = op.join(reg_dir, "mean_func_xfm.nii.gz")
+        reg = op.join(reg_dir, "func2anat_tkreg.dat")
     ffx_templates["anatomy"] = bg
     ffx_templates["reg_file"] = reg
 
@@ -446,13 +479,13 @@ def parse_args(arglist):
         with the experiment details defined in $LYMAN_DIR/nback-parametric.py.
         This assumes preprocessing has been performed for the nback experiment.
 
-    run_fmri.py -w preproc reg -t -u -r epi
+    run_fmri.py -w preproc reg -t -u -reg epi
 
         Preprocess the default experiment for all subjects, and then align
         the unsmoothed timeseries into the epi space. This is the standard set
         of processing that must be performed before multivariate analyses.
 
-    run_fmri.py -w reg ffx -r epi
+    run_fmri.py -w reg ffx -reg epi
 
         Align the summary statistics for all subjects into the epi space and
         then combine across runs. This is the standard processing that must
@@ -478,8 +511,12 @@ def parse_args(arglist):
                         help="which workflows to run")
     parser.add_argument("-regspace", default="mni", choices=wf.spaces,
                         help="common space for registration and fixed effects")
+    parser.add_argument("-regexp",
+                        help="perform cross-experiment epi registration")
     parser.add_argument("-timeseries", action="store_true",
                         help="perform registration on preprocessed timeseries")
+    parser.add_argument("-residual", action="store_true",
+                        help="perform registration on residual timeseries")
     parser.add_argument("-unsmoothed", action="store_true",
                         help="used unsmoothed data for model, reg, and ffx")
     return parser.parse_args(arglist)
